Unit LibSDL2;

// Dynamic SDL2 interface for FreePascal

{$MODE DELPHI}

Interface

Uses
  DynLibs;

{$DEFINE USECDECL}

Const
  SDL_INIT_VIDEO = $00000020;

Type
  SInt32 = LongInt;

  TSDL_Init = Function (Flags: UInt32) : SInt32; {$IFDEF USECDECL} CDECL; {$ELSE} STDCALL; {$ENDIF}
  TSDL_Quit = Procedure; {$IFDEF USECDECL} CDECL; {$ELSE} STDCALL; {$ENDIF}

Var
  SDL_Init      : TSDL_Init;
  SDL_Quit      : TSDL_Quit;

  SDL2LoadError : Boolean = False;
  SDL2Handle    : TLibHandle;
  IsSDL2Loaded  : Boolean;

Function LoadAllSDL2 : Boolean;
Function LoadSDL2 (Const LibName: RawByteString) : Boolean;

Implementation

Function Load (Const N: String) : Pointer;
Begin
  {$IFDEF DARWIN}
    Result := GetProcAddress(SDL2Handle, '_' + N);
  {$ELSE}
    Result := GetProcAddress(SDL2Handle, N);
  {$ENDIF}

  If Result = NIL Then Begin
    WriteLn ('SDL2 load error: Name=', N, ' Msg=', GetLoadErrorStr);

    SDL2LoadError := True;
  End;
End;

Function LoadSDL2 (Const LibName: RawByteString) : Boolean;
Begin
  If IsSDL2Loaded Then Exit(True);

  SDL2Handle   := LoadLibrary(LibName);
  IsSDL2Loaded := SDL2Handle <> 0;

  If IsSDL2Loaded Then Begin
    // Load functions here

    @SDL_Init := Load('SDL_Init');
    @SDL_Quit := Load('SDL_Quit');

    Result := SDL2LoadError;

    If SDL2LoadError Then Begin
      WriteLn ('Error loading SDL2 library');

      UnloadLibrary(SDL2Handle);

      IsSDL2Loaded := False;
    End;
  End;

  Result := IsSDL2Loaded;
End;

Function LoadAllSDL2 : Boolean;
// Attempt to automatically find SDL2 based on usual locations
Begin
  Result := IsSDL2Loaded;

  If IsSDL2Loaded Then Exit;

  {$IFDEF WINDOWS}
    {$IFDEF CPU64}
      If Not LoadSDL2('SDL264.DLL') Then
        If Not LoadSDL2('lib\SDL264.DLL') Then
          If Not LoadSDL2('SDL2.DLL') Then
            LoadSDL2('lib\SDL2.DLL');
    {$ENDIF}

    {$IFDEF CPU32}
      If Not LoadSDL2('SDL232.DLL') Then
        If Not LoadSDL2('lib\SDL232.DLL') Then
          If Not LoadSDL2('SDL2.DLL') Then
            LoadSDL2('lib\SDL2.DLL');
    {$ENDIF}
  {$ENDIF}

  {$IFDEF LINUX}
    {$IFDEF CPU64}
      If Not LoadSDL2('libSDL2.so') Then
        If Not LoadSDL2('/usr/lib/x86_64-linux-gnu/libSDL2.so') Then
          If Not LoadSDL2('/usr/local/lib/libSDL2.so') Then
            LoadSDL2('/usr/lib/libSDL2.so');
    {$ENDIF}

    {$IFDEF CPU32}
      If Not LoadSDL2('libSDL2.so') Then
        If Not LoadSDL2('/usr/lib/i386-linux-gnu/libSDL2.so') Then
          If Not LoadSDL2('/usr/local/lib/libSDL2.so') Then
            LoadSDL2('/usr/lib/libSDL2.so');
    {$ENDIF}

    {$IFDEF CPUARM}
      If Not LoadSDL2('libSDL2.so') Then
        If Not LoadSDL2('/usr/local/lib/libSDL2.so') Then
          LoadSDL2('/usr/lib/libSDL2.so');
    {$ENDIF}

    {$IFDEF CPUAARCH64}
      If Not LoadSDL2('libSDL2.so') Then
        If Not LoadSDL2('/usr/local/lib/libSDL2.so') Then
          LoadSDL2('/usr/lib/libSDL2.so');
    {$ENDIF}
  {$ENDIF}

  {$IFDEF DARWIN}
    LoadSDL2('libSDL2.dylib');
  {$ENDIF}

  Result:= IsSDL2Loaded;
End;

Procedure FreeSDL2Library;
Begin
  If IsSDL2Loaded Then Begin
    UnloadLibrary(SDL2Handle);

    IsSDL2Loaded := False;
  End;
End;

Initialization

  SDL2Handle   := 0;
  IsSDL2Loaded := False;

Finalization

  FreeSDL2Library;

End.
