Program TestSDL;

// Headers are from the link and extracted in SDL2 subdirectory
// https://github.com/ev1313/Pascal-SDL-2-Headers/archive/refs/heads/master.zip

{$MODE           DELPHI}
{$DEBUGINFO      OFF}
{$SMARTLINK      ON}
{$OVERFLOWCHECKS OFF}
{$CHECKPOINTER   OFF}
{$OPTIMIZATION   LEVEL3}
{$S-}

Uses
  {$IFDEF UNIX}
    CThreads,
  {$ENDIF}
  SysUtils,
  SDL2 in 'SDL2/sdl2.pas';

Var
  Window   : PSDL_Window;
  Renderer : PSDL_Renderer;
  Event    : TSDL_Event;
  Flags    : LongInt;
  Rect     : TSDL_Rect;
  Frame    : LongInt;
Begin
  WriteLn ('Start TestSDL');

  If SDL_Init(SDL_INIT_VIDEO) < 0 Then Begin
    WriteLn ('Error initializing video: ', SDL_GetError);

    Exit;
  End;

  Flags  := SDL_WINDOW_SHOWN OR SDL_WINDOW_RESIZABLE;
  Window := SDL_CreateWindow('Test SDL',
              SDL_WINDOWPOS_CENTERED,
              SDL_WINDOWPOS_CENTERED,
              640, 480,
              Flags);

  If Window = NIL Then Begin
    WriteLn('Error creating window');

    Halt;
  End;

  Renderer := SDL_CreateRenderer(Window, -1, SDL_RENDERER_ACCELERATED);

  If Renderer = NIL Then Begin
    WriteLn('Error creating renderer');

    SDL_DestroyWindow(Window);
    SDL_Quit;

    Halt;
  End;

  Try
    SDL_RenderClear   (Renderer);
    SDL_RenderPresent (Renderer);
  Except
    On E: Exception Do
    //On E: EDivByZero Do
      Begin
        WriteLn ('Error using GPU-accelerated drivers, falling back to (slow) software renderer');

        SDL_DestroyRenderer(Renderer);

        Renderer := SDL_CreateRenderer(Window, -1, SDL_RENDERER_SOFTWARE);

        If Renderer = NIL Then Begin
          WriteLn('Error creating renderer');

          SDL_DestroyWindow(Window);
          SDL_Quit;

          Halt;
        End;
      End;
  End;

  Frame := 1;

  While True Do Begin
    If SDL_PollEvent(@Event) = 1 Then Begin
      If Event.Type_ = SDL_QUITEV Then
        Break;
    End;

    SDL_Delay(50);

    Inc (Frame);

    If Frame = 10 Then Begin
      SDL_SetRenderDrawColor(Renderer, Random(255), Random(255), Random(255), 255);

      Rect.X := 100;
      Rect.Y := 100;
      Rect.W := 100;
      Rect.H := 100;

      SDL_RenderFillRect(Renderer, @Rect);

      Frame := 1;
    End;

    SDL_RenderClear   (Renderer);
    SDL_RenderPresent (Renderer);
  End;

  SDL_DestroyRenderer(Renderer);
  SDL_DestroyWindow(Window);
  SDL_Quit;

  WriteLn ('TestSDL complete');
End.
z
